@@ -34,6 +34,8 @@
 #include "e2k-restriction.h"
 #include "e2k-utils.h"
 
+#define d(x)
+
 /* Placeholder for each component and its recurrences */
 typedef struct {
         ECalComponent *full_object;
@@ -806,7 +808,12 @@ open_task (ECalBackendSync *backend, EDataCal *cal,
 	status = E_CAL_BACKEND_SYNC_CLASS (parent_class)->open_sync (backend,
 				     cal, only_if_exits, username, password);
 	if (status != GNOME_Evolution_Calendar_Success)
-		return GNOME_Evolution_Calendar_OtherError;
+		return status;
+
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
+		d(printf ("ECBEC : calendar is offline\n"));
+		return GNOME_Evolution_Calendar_Success;
+	}
 
 	status = get_changed_tasks (E_CAL_BACKEND_EXCHANGE (backend), NULL);
 	if (status != E2K_HTTP_OK)
@@ -847,6 +854,12 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 	ecalbex = E_CAL_BACKEND_EXCHANGE (backend);
 
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
+
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
+		d(printf ("tasks are offline\n"));
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+
 	/* Parse the icalendar text */
 	icalcomp = icalparser_parse_string (*calobj);
 	if (!icalcomp)
@@ -937,7 +950,7 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 static ECalBackendSyncStatus
 modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	       const char *calobj, CalObjModType mod,
-	       char **old_object)
+	       char **old_object, char **new_object)
 {
 	ECalBackendExchangeTasks *ecalbextask;
 	ECalBackendExchangeComponent *ecalbexcomp;
@@ -959,6 +972,11 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	g_return_val_if_fail (calobj != NULL, 
 				GNOME_Evolution_Calendar_ObjectNotFound);
 
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
+		d(printf ("tasks are offline\n"));
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+
 	/* Parse the icalendar text */
         icalcomp = icalparser_parse_string ((char *) calobj);
         if (!icalcomp)
@@ -984,7 +1002,7 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	}
         
         cache_comp = e_cal_component_new ();
-        e_cal_component_set_icalcomponent (cache_comp, ecalbexcomp->comp);
+        e_cal_component_set_icalcomponent (cache_comp, ecalbexcomp->icomp);
 	*old_object = e_cal_component_get_as_string (cache_comp);
 	g_free (cache_comp);
 	
@@ -1041,6 +1059,11 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
         g_return_val_if_fail (calobj != NULL,
                                 GNOME_Evolution_Calendar_ObjectNotFound);
                                                                                 
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
+		d(printf ("tasks are offline\n"));
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+
 	status = e_cal_backend_exchange_extract_components (calobj, &method, &comps);
         if (status != GNOME_Evolution_Calendar_Success)
                 return GNOME_Evolution_Calendar_InvalidObject;
@@ -1067,7 +1090,7 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
                                                                                 
                 if (get_exchange_comp (E_CAL_BACKEND_EXCHANGE (ecalbextask), uid)) {
                         char *old_object;
-                        status = modify_task_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object);
+                        status = modify_task_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object, NULL);
                         if (status != GNOME_Evolution_Calendar_Success)
                                 goto error;
                                                                                 
@@ -1092,7 +1115,7 @@ error:
 static ECalBackendSyncStatus
 remove_task_object (ECalBackendSync *backend, EDataCal *cal,
 	       const char *uid, const char *rid, CalObjModType mod,
-	       char **old_object)
+	       char **old_object, char **object)
 {
 	ECalBackendExchange *ecalbex = E_CAL_BACKEND_EXCHANGE (backend);
 	ECalBackendExchangeComponent *ecalbexcomp;
@@ -1103,12 +1126,17 @@ remove_task_object (ECalBackendSync *backend, EDataCal *cal,
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (ecalbex), 
 					GNOME_Evolution_Calendar_OtherError);
 
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
+		d(printf ("tasks are offline\n"));
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+
 	ecalbexcomp = get_exchange_comp (ecalbex, uid);
 	if (!ecalbexcomp && !ecalbexcomp->href)
 		return GNOME_Evolution_Calendar_ObjectNotFound;		
 
         comp = e_cal_component_new ();
-        e_cal_component_set_icalcomponent (comp, ecalbexcomp->comp);
+        e_cal_component_set_icalcomponent (comp, ecalbexcomp->icomp);
 	*old_object = e_cal_component_get_as_string (comp);
 
 	ctx = exchange_account_get_context (ecalbex->account);
