@@ -27,6 +27,7 @@
 
 #include "e-cal-backend-exchange.h"
 #include "e2k-cal-utils.h"
+#include "e2k-uri.h"
 
 #include "e2k-propnames.h"
 #include "e2k-restriction.h"
@@ -48,6 +49,7 @@ struct ECalBackendExchangePrivate {
 	/* Timezones */
 	GHashTable *timezones;
 	icaltimezone *default_timezone;
+	CalMode mode;
 };
 
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_SYNC
@@ -72,9 +74,14 @@ get_cal_address (ECalBackendSync *backend, EDataCal *cal, char **address)
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	ExchangeHierarchy *hier;
 
-	hier = e_folder_exchange_get_hierarchy (cbex->folder);
-	d(printf("ecbe_get_cal_address(%p, %p) -> %s\n", backend, cal, hier->owner_email));
-	*address = g_strdup (hier->owner_email);
+	if (cbex->priv->mode == CAL_MODE_REMOTE) {
+		hier = e_folder_exchange_get_hierarchy (cbex->folder);
+		d(printf("ecbe_get_cal_address(%p, %p) -> %s\n", backend, cal, hier->owner_email));
+		*address = g_strdup (hier->owner_email);
+	} else {
+		*address = NULL;  /* FIXME : For offline, fetch this from gconf */
+	}
+
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -110,15 +117,17 @@ get_static_capabilities (ECalBackendSync *backend, EDataCal *cal, char **capabil
 		CAL_STATIC_CAPABILITY_NO_TASK_ASSIGNMENT ","
 		CAL_STATIC_CAPABILITY_NO_THISANDFUTURE ","
 		CAL_STATIC_CAPABILITY_NO_THISANDPRIOR ","
+		CAL_STATIC_CAPABILITY_NO_GEN_OPTIONS ","
+		CAL_STATIC_CAPABILITY_REQ_SEND_OPTIONS ","
 		CAL_STATIC_CAPABILITY_REMOVE_ALARMS);
 
 	return GNOME_Evolution_Calendar_Success;
 }
 
 static void
-load_cache (ECalBackendExchange *cbex)
+load_cache (ECalBackendExchange *cbex, E2kUri *e2kuri)
 {
-	icalcomponent *vcalcomp, *comp;
+	icalcomponent *vcalcomp, *comp, *tmp_comp;
 	struct icaltimetype comp_last_mod, folder_last_mod;
 	icalcomponent_kind kind;
 	icalproperty *prop;
@@ -158,7 +167,11 @@ load_cache (ECalBackendExchange *cbex)
 	for (comp = icalcomponent_get_first_component (vcalcomp, ICAL_VTIMEZONE_COMPONENT);
 	     comp;
 	     comp = icalcomponent_get_next_component (vcalcomp, ICAL_VTIMEZONE_COMPONENT)) {
-		e_cal_backend_exchange_add_timezone (cbex, icalcomponent_new_clone (comp));
+		tmp_comp = icalcomponent_new_clone (comp);
+		if (tmp_comp) {
+			e_cal_backend_exchange_add_timezone (cbex, tmp_comp);
+			icalcomponent_free (tmp_comp);
+		}
 	}
 
 	icalcomponent_free (vcalcomp);
@@ -180,7 +193,7 @@ save_object (gpointer key, gpointer value, gpointer vcalcomp)
 	icalcomponent *icalcomp;
 	GList *l;
 
-	icalcomp = icalcomponent_new_clone (ecomp->comp);
+	icalcomp = icalcomponent_new_clone (ecomp->icomp);
 	icalcomponent_add_component (vcalcomp, icalcomp);
 
 	for (l = ecomp->instances; l; l = l->next) {
@@ -209,20 +222,17 @@ timeout_save_cache (gpointer user_data)
 
 	tmpfile = g_strdup_printf ("%s~", cbex->priv->object_cache_file);
 	f = fopen (tmpfile, "w");
-	if (!f) {
-		g_free (tmpfile);
-		return FALSE;
-	}
+	if (!f)
+		goto error;
 
 	len = strlen (data);
 	nwrote = fwrite (data, 1, len, f);
-	if (fclose (f) != 0 || nwrote != len) {
-		g_free (tmpfile);
-		return FALSE;
-	}
+	if (fclose (f) != 0 || nwrote != len)
+		goto error;
 
 	if (rename (tmpfile, cbex->priv->object_cache_file) != 0)
 		unlink (tmpfile);
+error:
 	g_free (tmpfile);
 	return FALSE;
 }
@@ -250,13 +260,22 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 	const char *prop = PR_ACCESS;
 	E2kHTTPStatus status;
 	E2kResult *results;
+	E2kUri *euri = NULL;
 	int nresults;
 	guint access = 0;
 
 	d(printf("ecbe_open_calendar(%p, %p, %sonly if exists, user=%s, pass=%s)\n", backend, cal, only_if_exists?"":"not ", username?username:"(null)", password?password:"(null)"));
 
-	/* Make sure we have an open connection */
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
+	if (cbex->priv->mode == CAL_MODE_LOCAL) {
+		d(printf ("ECBE : cal is offline .. load cache\n"));
+		euri = e2k_uri_new (uristr);
+		load_cache (cbex, euri);
+		e2k_uri_free (euri);
+		return GNOME_Evolution_Calendar_Success;
+	}
+		
+	/* Make sure we have an open connection */
 	cbex->account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
 	if (!cbex->account)
 		return GNOME_Evolution_Calendar_RepositoryOffline;
@@ -294,7 +313,7 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 
 	cbex->priv->read_only = ((access & MAPI_ACCESS_CREATE_CONTENTS) == 0);
 
-	load_cache (cbex);
+	load_cache (cbex, euri);
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -303,7 +322,9 @@ static ECalBackendSyncStatus
 remove_calendar (ECalBackendSync *backend, EDataCal *cal)
 {
 	d(printf("ecbe_remove_calendar(%p, %p)\n", backend, cal));
-	return GNOME_Evolution_Calendar_PermissionDenied;
+
+	//display_error_dialog("Can not perform this operation, Use Exchange component for folder operations\n");
+	return GNOME_Evolution_Calendar_PermissionDenied; /* Error code is not handled in Evolution */
 
 	/* FIXME: Deleting calendar/tasks from respective views */
 #if 0
@@ -404,6 +425,7 @@ e_cal_backend_exchange_add_object (ECalBackendExchange *cbex,
 	ECalBackendExchangeComponent *ecomp;
 	gboolean is_instance;
 	const char *uid;
+	struct icaltimetype rid;
 
 	d(printf("ecbe_add_object(%p, %s, %s)\n", cbex, href, lastmod));
 
@@ -412,7 +434,7 @@ e_cal_backend_exchange_add_object (ECalBackendExchange *cbex,
 
 	is_instance = (icalcomponent_get_first_property (comp, ICAL_RECURRENCEID_PROPERTY) != NULL);
 
-	if (ecomp && ecomp->comp && !is_instance)
+	if (ecomp && ecomp->icomp && !is_instance)
 		return FALSE;
 
 	if (!ecomp) {
@@ -431,10 +453,25 @@ e_cal_backend_exchange_add_object (ECalBackendExchange *cbex,
 	}
 
 	if (is_instance) {
-		ecomp->instances = g_list_prepend (ecomp->instances,
+		GList *l;
+		struct icaltimetype inst_rid;
+		gboolean inst_found = FALSE;
+
+		rid = icalcomponent_get_recurrenceid (comp);
+		for (l = ecomp->instances; l; l = l->next) {
+			inst_rid = icalcomponent_get_recurrenceid (l->data);
+			if (icaltime_compare (inst_rid, rid) == 0) {
+				inst_found = TRUE;
+				break;
+			}
+		}
+		if (!inst_found) {
+
+			ecomp->instances = g_list_prepend (ecomp->instances,
 						   icalcomponent_new_clone (comp));
+		}
 	} else
-		ecomp->comp = icalcomponent_new_clone (comp);
+		ecomp->icomp = icalcomponent_new_clone (comp);
 
 	save_cache (cbex);
 	return TRUE;
@@ -450,8 +487,8 @@ discard_detached_instance (ECalBackendExchangeComponent *ecomp,
 	for (inst = ecomp->instances; inst; inst = inst->next) {
 		inst_rid = icalcomponent_get_recurrenceid (inst->data);
 		if (icaltime_compare (inst_rid, rid) == 0) {
-			icalcomponent_free (inst->data);
 			ecomp->instances = g_list_remove (ecomp->instances, inst->data);
+			icalcomponent_free (inst->data);
 			return;
 		}
 	}
@@ -490,8 +527,8 @@ e_cal_backend_exchange_modify_object (ECalBackendExchange *cbex,
 		return FALSE;
 
 	if (mod == CALOBJ_MOD_ALL || icaltime_is_null_time (rid)) {
-		icalcomponent_free (ecomp->comp);
-		ecomp->comp = icalcomponent_new_clone (comp);
+		icalcomponent_free (ecomp->icomp);
+		ecomp->icomp = icalcomponent_new_clone (comp);
 	} else {
 		discard_detached_instance (ecomp, rid);
 		ecomp->instances = g_list_prepend (ecomp->instances,
@@ -544,7 +581,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 /*To be overriden by Calendar and Task classes*/
 static ECalBackendSyncStatus
 modify_object (ECalBackendSync *backend, EDataCal *cal, 
-			const char * calobj, CalObjModType mod, char **old_object)
+			const char * calobj, CalObjModType mod, char **old_object, char **new_object)
 {
 	return GNOME_Evolution_Calendar_OtherError;
 }
@@ -576,15 +613,21 @@ get_object (ECalBackendSync *backend, EDataCal *cal,
 	
 	ecomp = g_hash_table_lookup (cbex->priv->objects, uid);
 	if (!ecomp)
+	if ((!ecomp) || (!ecomp->icomp))
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	
 	/*anything on recur id here????*/
 	
 	comp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (comp, ecomp->comp);
-
-	*object = e_cal_component_get_as_string (comp);
+	if (e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp))) {
+		*object = e_cal_component_get_as_string (comp);
+	} else {
+		*object = NULL;
+		g_object_unref (comp);
+		return GNOME_Evolution_Calendar_ObjectNotFound;
+	}
 	
+	g_object_unref (comp);
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -668,11 +711,14 @@ static ECalBackendSyncStatus
 get_default_object (ECalBackendSync *backend, EDataCal *cal, char **object)
 {
 	icalcomponent *comp;
+	char *ical_obj;
 
 	d(printf("ecbe_get_default_object(%p, %p)\n", backend, cal));
 
 	comp = e_cal_util_new_component (e_cal_backend_get_kind (E_CAL_BACKEND (backend)));
-	*object = g_strdup (icalcomponent_as_ical_string (comp));
+	ical_obj = icalcomponent_as_ical_string (comp);
+	*object = g_strdup (ical_obj);
+
 	icalcomponent_free (comp);
 
 	return GNOME_Evolution_Calendar_Success;
@@ -695,7 +741,7 @@ match_object_sexp (gpointer key, gpointer value, gpointer data)
 	ECalComponent *comp;
 	
 	comp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (comp, ecomp->comp);
+	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp));
 
 	if ((!match_data->search_needed) ||
 	    (e_cal_backend_sexp_match_comp (match_data->obj_sexp, comp, match_data->backend))) {
@@ -709,6 +755,7 @@ match_object_sexp (gpointer key, gpointer value, gpointer data)
 				      match_data);
 		#endif
 	}
+	g_object_unref (comp);
 }
 
 static ECalBackendSyncStatus
@@ -748,6 +795,7 @@ get_timezone (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	icaltimezone *zone;
 	icalcomponent *vtzcomp;
+	char *ical_obj;
 
 	d(printf("ecbe_get_timezone(%p, %p, %s)\n", backend, cal, tzid));
 
@@ -759,7 +807,9 @@ get_timezone (ECalBackendSync *backend, EDataCal *cal,
 	if (!vtzcomp)
 		return GNOME_Evolution_Calendar_OtherError;
 
-	*object = g_strdup (icalcomponent_as_ical_string (vtzcomp));
+	ical_obj = icalcomponent_as_ical_string (vtzcomp);
+	*object = g_strdup (ical_obj);
+
 	return  GNOME_Evolution_Calendar_Success;
 }
 
@@ -816,15 +866,17 @@ add_timezone (ECalBackendSync *backend, EDataCal *cal,
 	status = e_cal_backend_exchange_add_timezone (cbex, vtzcomp);
 	switch (status) {
 	case GNOME_Evolution_Calendar_ObjectIdAlreadyExists:
-		icalcomponent_free (vtzcomp);
 		/* fall through */
 
 	case GNOME_Evolution_Calendar_Success:
+		icalcomponent_free (vtzcomp);
 		return GNOME_Evolution_Calendar_Success;
 
 	default:
+		icalcomponent_free (vtzcomp);
 		return status;
 	}
+	icalcomponent_free (vtzcomp);
 }
 
 static ECalBackendSyncStatus
@@ -856,21 +908,30 @@ match_object (gpointer key, gpointer value, gpointer data)
 	ECalBackendExchangeComponent *ecomp = value;
 	struct search_data *sd = data;
 	ECalComponent *cal_comp;
+	icalcomponent *tmp;
+	char * ecal_str;
 	GList *inst;
 
 	/* FIXME: we shouldn't have to convert to ECalComponent here */
 	cal_comp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (cal_comp, icalcomponent_new_clone (ecomp->comp));
-	if (e_cal_backend_sexp_match_comp (sd->sexp, cal_comp, sd->backend)) {
-		sd->matches = g_list_prepend (sd->matches,
-					      e_cal_component_get_as_string (cal_comp));
+
+	/* Find a way to test if the icalcomp added to cal_comp is not null */
+	tmp = icalcomponent_new_clone (ecomp->icomp);
+	if (tmp) {
+		e_cal_component_set_icalcomponent (cal_comp, tmp);
+		if (e_cal_backend_sexp_match_comp (sd->sexp, cal_comp, sd->backend)) {
+			ecal_str = e_cal_component_get_as_string (cal_comp);
+			if (ecal_str)
+				sd->matches = g_list_prepend (sd->matches, ecal_str);
+		}
 	}
 
 	for (inst = ecomp->instances; inst; inst = inst->next) {
 		e_cal_component_set_icalcomponent (cal_comp, icalcomponent_new_clone (inst->data));
 		if (e_cal_backend_sexp_match_comp (sd->sexp, cal_comp, sd->backend)) {
-			sd->matches = g_list_prepend (sd->matches,
-						      e_cal_component_get_as_string (cal_comp));
+			ecal_str = e_cal_component_get_as_string (cal_comp);
+			if (ecal_str)
+				sd->matches = g_list_prepend (sd->matches, ecal_str);
 		}
 	}
 
@@ -907,27 +968,86 @@ start_query (ECalBackend *backend, EDataCalView *view)
 	e_data_cal_view_notify_done (view, GNOME_Evolution_Calendar_Success);
 }
 
+gboolean 
+e_cal_backend_exchange_is_online (ECalBackendExchange *cbex)
+{
+	if (cbex->priv->mode == CAL_MODE_LOCAL)
+		return FALSE;
+	else
+		return TRUE;
+}
+
 static CalMode
 get_mode (ECalBackend *backend)
 {
+	ECalBackendExchange *cbex;
+	ECalBackendExchangePrivate *priv;
+	
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	priv = cbex->priv;
+
 	d(printf("ecbe_get_mode(%p)\n", backend));
 
-	return CAL_MODE_REMOTE;
+	return priv->mode;
 }
 
 static void
 set_mode (ECalBackend *backend, CalMode mode)
 {
-	d(printf("ecbe_set_mode(%p)\n", backend));
+	ECalBackendExchange *cbex;
+	ECalBackendExchangePrivate *priv;
+	const char *uristr;
+	
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	priv = cbex->priv;
 
-	if (mode == CAL_MODE_REMOTE) {
+	d(printf("ecbe_set_mode(%p) : mode : %d\n", backend, mode));
+
+	if (priv->mode == mode) {
 		e_cal_backend_notify_mode (
 			backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
-			GNOME_Evolution_Calendar_MODE_REMOTE);
-	} else {
+			cal_mode_to_corba (mode));
+	}
+
+	switch (mode) {
+	
+	case CAL_MODE_REMOTE:
+			/* Change status to be online now */
+#if 0
+			if (exchange_account_set_online (cbex->account)) { printf ("mode set to online\n"); 
+				priv->mode = CAL_MODE_REMOTE;
+			}
+			else { printf ("failed to set mode to online\n");
+				break;
+			}
+#endif
+			/* Should we check for access rights before setting this ? */
+			priv->read_only = FALSE;
+			e_cal_backend_notify_mode (backend, 
+				GNOME_Evolution_Calendar_CalListener_MODE_SET,
+				GNOME_Evolution_Calendar_MODE_REMOTE);
+			/* FIXME : Check if online and check if authentication
+				is needed */
+			break;
+
+	case CAL_MODE_LOCAL:
+			/* FIXME : Update the cache before closing the connection */
+			d(printf ("set mode to offline\n"));
+			uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
+			cbex->account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
+			cbex->folder = exchange_account_get_folder (cbex->account, uristr);
+			priv->mode = CAL_MODE_LOCAL;
+			/* FIXME : Set connection to NULL and become offline */
+			exchange_account_set_offline (cbex->account);
+			e_cal_backend_notify_mode (backend, 
+				GNOME_Evolution_Calendar_CalListener_MODE_SET,
+				GNOME_Evolution_Calendar_MODE_LOCAL);
+			break;
+
+	default :
 		e_cal_backend_notify_mode (
 			backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED,
-			GNOME_Evolution_Calendar_MODE_REMOTE);
+			cal_mode_to_corba (mode));
 	}
 }
 
@@ -1031,9 +1151,13 @@ e_cal_backend_exchange_get_cal_address (ECalBackendSync *backend)
 	ExchangeHierarchy *hier;
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex), NULL);
-
-	hier = e_folder_exchange_get_hierarchy (cbex->folder);
-	return hier->owner_email;
+	if (cbex->priv->mode == CAL_MODE_REMOTE) {
+		hier = e_folder_exchange_get_hierarchy (cbex->folder);
+		//cbex->priv->email_address = g_strdup (hier->owner_email);
+		return hier->owner_email;
+	} else {
+		return NULL; /* FIXME : Read this from gconf */
+	}
 }
 
 const char *
@@ -1087,7 +1211,7 @@ check_change_type (gpointer key, gpointer value, gpointer data)
 	if (ecomp != NULL) {
 
 		comp = e_cal_component_new ();
-		e_cal_component_set_icalcomponent (comp, ecomp->comp);
+		e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp));
 
 		calobj = e_cal_component_get_as_string (comp);
 		switch (e_xmlhash_compare (change_data->ehash, uid, calobj)){
@@ -1103,6 +1227,7 @@ check_change_type (gpointer key, gpointer value, gpointer data)
 		}
 		
 		g_free (calobj);
+		g_object_unref (comp);
 	}
 		
 }
@@ -1132,7 +1257,7 @@ e_cal_backend_exchange_compute_changes_foreach_key (const char *key, gpointer da
 		cbedata->deletes = g_list_prepend (cbedata->deletes, e_cal_component_get_as_string (comp));
 		
 		e_xmlhash_remove (cbedata->ehash, key);
-		
+		g_object_unref (comp);	
 	}
 }
 
@@ -1189,8 +1314,6 @@ internal_get_default_timezone (ECalBackend *backend)
 {
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
-	/* d(printf("ecbe_internal_get_default_timezone(%p)\n", backend)); */
-
 	if (!cbex->priv->default_timezone &&
 	    cbex->account->default_timezone) {
 		cbex->priv->default_timezone =
@@ -1206,8 +1329,6 @@ internal_get_timezone (ECalBackend *backend, const char *tzid)
 {
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
-	/* d(printf("ecbe_internal_get_timezone(%p, %s)\n", backend, tzid)); */
-
 	return g_hash_table_lookup (cbex->priv->timezones, tzid);
 }
 
@@ -1222,7 +1343,7 @@ free_exchange_comp (gpointer value)
 	g_free (ecomp->href);
 	g_free (ecomp->lastmod);
 
-	icalcomponent_free (ecomp->comp);
+	icalcomponent_free (ecomp->icomp);
 
 	for (inst = ecomp->instances; inst; inst = inst->next)
 		icalcomponent_free (inst->data);
