@@ -32,6 +32,7 @@
 #include "exchange-hierarchy-favorites.h"
 #include "exchange-hierarchy-foreign.h"
 #include "exchange-hierarchy-gal.h"
+#include "exchange-offline-listener.h"
 #include "e-folder-exchange.h"
 #include "e2k-autoconfig.h"
 #include "e2k-encoding-utils.h"
@@ -46,11 +47,21 @@
 #include <gal/util/e-util.h>
 #include <libgnome/gnome-util.h>
 
+#include <glade/glade-xml.h>
+#include <gtk/gtkdialog.h>
+#include <gtk/gtklabel.h>
+
 #include <dirent.h>
 #include <stdlib.h>
 #include <string.h>
 
 #define d(x) x
+#define ADS_UF_DONT_EXPIRE_PASSWORD 0x10000
+#define ONE_HUNDRED_NANOSECOND 0.000000100
+#define SECONDS_IN_DAY 86400
+#define PASSWD_EXPIRY_NOTIFICATION_PERIOD 7
+#define FILENAME CONNECTOR_GLADEDIR "/exchange-passwd-expiry.glade"
+#define ROOTNODE "passwd_exp_dialog"
 
 struct _ExchangeAccountPrivate {
 	E2kContext *ctx;
@@ -58,7 +69,7 @@ struct _ExchangeAccountPrivate {
 	GHashTable *standard_uris;
 
 	GMutex *connect_lock;
-	gboolean connecting, connected;
+	gboolean connecting, connected, account_online;
 
 	GPtrArray *hierarchies;
 	GHashTable *hierarchies_by_folder, *foreign_hierarchies;
@@ -68,7 +79,7 @@ struct _ExchangeAccountPrivate {
 	gboolean uris_use_email;
 
 	char *identity_name, *identity_email, *source_uri, *password_key;
-	char *username, *password, *ad_server;
+	char *username, *password, *windows_domain, *ad_server;
 	E2kAutoconfigAuthPref auth_pref;
 	int ad_limit;
 
@@ -146,6 +157,7 @@ init (GObject *object)
 	account->priv->foreign_hierarchies = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->folders = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->discover_data_lock = g_mutex_new ();
+	account->priv->account_online = TRUE;
 }
 
 static void
@@ -267,6 +279,9 @@ finalize (GObject *object)
 			strlen (account->priv->password));
 		g_free (account->priv->password);
 	}
+	if (account->priv->windows_domain)
+		g_free (account->priv->windows_domain);
+
 	if (account->priv->ad_server)
 		g_free (account->priv->ad_server);
 
@@ -878,6 +893,98 @@ is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 	return (result == E2K_KERBEROS_PASSWORD_EXPIRED);
 }
 
+static void
+change_passwd_cb (GtkWidget *button, char *old_passwd)
+{
+	gtk_widget_hide (gtk_widget_get_toplevel(button));
+	exchange_get_new_password (old_passwd, TRUE);
+}
+
+static void
+display_passwd_expiry_message (int max_passwd_age, char *old_passwd)
+{
+	GladeXML *xml;
+	GtkWidget *top_widget, *change_passwd_button;
+	GtkResponseType response;
+	GtkLabel *warning_msg_label;
+	char *passwd_expiry_msg = 
+		g_strdup_printf ("Your password will expire in next %d days\n",
+				  max_passwd_age);
+
+	xml = glade_xml_new (FILENAME, ROOTNODE, NULL);
+	g_return_if_fail (xml != NULL);
+	top_widget = glade_xml_get_widget (xml, ROOTNODE);
+	g_return_if_fail (top_widget != NULL);
+
+	warning_msg_label = GTK_LABEL (glade_xml_get_widget (xml, 
+						"passwd_exp_label"));
+	gtk_label_set_text (warning_msg_label, passwd_expiry_msg);
+
+	change_passwd_button = glade_xml_get_widget (xml, 
+						"change_passwd_button");
+	gtk_widget_set_sensitive (change_passwd_button, TRUE);
+	g_signal_connect (change_passwd_button, 
+			  "clicked", 
+			  G_CALLBACK (change_passwd_cb), 
+			  old_passwd);
+
+	response = gtk_dialog_run (GTK_DIALOG (top_widget));
+
+	gtk_widget_destroy (top_widget);
+	g_object_unref (xml);
+	g_free (passwd_expiry_msg);
+}
+
+static void
+find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
+{
+	double max_pwd_age = 0;
+	int max_pwd_age_days;
+	E2kOperation gcop;
+	E2kGlobalCatalogStatus gcstatus;
+	char *current_passwd;
+
+	/* Check for password expiry period */ 
+	/* This needs to be invoked after is_password_expired(), i.e., 
+	   only if password is not expired */
+
+	/* Check for account control value for a user */
+
+	e2k_operation_init (&gcop);
+	gcstatus = e2k_global_catalog_lookup (account->priv->gc, 
+					      &gcop, 
+					      E2K_GLOBAL_CATALOG_LOOKUP_BY_EMAIL, 
+					      account->priv->identity_email, 
+					      E2K_GLOBAL_CATALOG_LOOKUP_ACCOUNT_CONTROL, 
+					      &entry); 
+	e2k_operation_free (&gcop);
+	if (gcstatus != E2K_GLOBAL_CATALOG_OK) 
+		return;
+       
+	if (entry->user_account_control & ADS_UF_DONT_EXPIRE_PASSWORD) 
+		return;         /* Password is not set to expire */
+
+	/* Here we don't check not setting the password and expired password */ 
+	/* Check for the maximum password age set */
+
+	e2k_operation_init (&gcop); 
+	max_pwd_age = lookup_passwd_max_age (account->priv->gc, &gcop); 
+	e2k_operation_free (&gcop);
+
+	if (max_pwd_age > 0) {
+		/* Calculate password expiry period */
+		max_pwd_age_days = 
+		( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
+
+		if (max_pwd_age_days <= PASSWD_EXPIRY_NOTIFICATION_PERIOD) {
+			/* This we need for changing password */
+			current_passwd = exchange_account_get_password (account);
+			display_passwd_expiry_message (max_pwd_age_days, 
+						       current_passwd);
+		}
+	} 
+}
+
 char *
 exchange_account_get_password (ExchangeAccount *account)
 {
@@ -932,6 +1039,57 @@ exchange_account_set_password (ExchangeAccount *account, char *old_pass, char *n
 	}
 }
 
+/**
+ * exchange_account_set_offline:
+ * @account: an #ExchangeAccount
+ *
+ * This nullifies the connection and sets the account as offline.
+ * The caller should take care that the required data is fetched
+ * before calling this method.
+ *
+ * Return value: Returns TRUE is successfully sets the account to
+ * offline or FALSE if failed
+ **/
+gboolean
+exchange_account_set_offline (ExchangeAccount *account)
+{
+		
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
+
+	if (account->priv->ctx) {
+		g_object_unref (account->priv->ctx);
+		account->priv->ctx = NULL;
+	}
+
+	account->priv->account_online = FALSE;
+	return TRUE;
+}
+
+/**
+ * exchange_account_set_online:
+ * @account: an #ExchangeAccount
+ *
+ * This nullifies the connection and sets the account as offline.
+ * The caller should take care that the required data is fetched
+ * before calling this method.
+ *
+ * Return value: Returns TRUE is successfully sets the account to
+ * offline or FALSE if failed
+ **/
+gboolean
+exchange_account_set_online (ExchangeAccount *account)
+{
+		
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
+
+	account->priv->account_online = TRUE;
+
+	if (exchange_account_connect (account))
+		return TRUE;
+	else
+		return FALSE;
+}
+
 /**
  * exchange_account_connect:
  * @account: an #ExchangeAccount
@@ -965,14 +1123,12 @@ exchange_account_connect (ExchangeAccount *account)
 	E2kGlobalCatalogEntry *entry;
 	E2kOperation gcop;
 	const char *quota_msg = NULL;
-	GtkWidget *quota_dialog;
-	int quota_resp;
+	char *user_name = NULL;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
 	g_mutex_lock (account->priv->connect_lock);
-
-	if (account->priv->connecting) {
+	if ((account->priv->connecting) || (!account->priv->account_online)){
 		g_mutex_unlock (account->priv->connect_lock);
 		return NULL;
 	} else if (account->priv->ctx) {
@@ -983,10 +1139,17 @@ exchange_account_connect (ExchangeAccount *account)
 	account->priv->connecting = TRUE;
 	g_mutex_unlock (account->priv->connect_lock);
 
+	if (account->priv->windows_domain)
+		user_name = g_strdup_printf ("%s\\%s", account->priv->windows_domain, account->priv->username);
+	else
+		user_name = g_strdup (account->priv->username);
+
 	ac = e2k_autoconfig_new (account->home_uri,
-				 account->priv->username,
+				 user_name,
 				 NULL,
 				 account->priv->auth_pref);
+	g_free (user_name);
+
 	e2k_autoconfig_set_gc_server (ac, account->priv->ad_server,
 				      account->priv->ad_limit);
 
@@ -1146,6 +1309,10 @@ exchange_account_connect (ExchangeAccount *account)
 		if (timezone)
 			account->default_timezone = g_strdup (timezone);
 	}
+
+	/* Find the password expiery peripod and display warning */
+	find_passwd_exp_period(account, entry);
+	
 	/* Check for quota warnings */
 	e2k_operation_init (&gcop);
 	gcstatus = e2k_global_catalog_lookup (account->priv->gc, &gcop,
@@ -1165,7 +1332,8 @@ exchange_account_connect (ExchangeAccount *account)
 			quota_msg = g_strdup_printf ("You are nearing your quota available for storing mails on this server. Your current usage is : %d . Try to clear up some space by deleting some mails.\n", entry->quota_warn);
 		}
 		
-		e_notice (NULL, GTK_MESSAGE_INFO, quota_msg);
+		if (quota_msg)
+			e_notice (NULL, GTK_MESSAGE_INFO, quota_msg);
 	}
 
 	/* Set up Personal Folders hierarchy */
@@ -1262,6 +1430,7 @@ exchange_account_connect (ExchangeAccount *account)
 	}
 	
 	account->priv->connected = TRUE;
+	account->priv->account_online = TRUE;
 	account->priv->connecting = FALSE;
 
 	g_signal_connect (account->priv->ctx, "redirect",
@@ -1537,6 +1706,10 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	account->priv->password_key = g_strdup_printf ("exchange://%s", account->priv->uri_authority);
 
 	account->priv->username = g_strdup (uri->user);
+	if (uri->domain)
+		account->priv->windows_domain = g_strdup (uri->domain);
+	else
+		account->priv->windows_domain = NULL;
 	account->exchange_server = g_strdup (uri->host);
 	if (uri->authmech && !strcmp (uri->authmech, "Basic"))
 		account->priv->auth_pref = E2K_AUTOCONFIG_USE_BASIC;
