@@ -58,6 +58,7 @@ struct ExchangeComponentPrivate {
 	EFolderTypeRegistry *folder_type_registry;
 
 	ExchangeConfigListener *config_listener;
+	ExchangeOfflineListener *offline_listener;
 	GSList *accounts;
 
 	GSList *views;
@@ -106,6 +107,11 @@ dispose (GObject *object)
 		priv->views = NULL;
 	}
 
+	if (priv->offline_listener) {
+		g_object_unref (priv->offline_listener);
+		priv->offline_listener = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -378,6 +384,18 @@ exchange_component_init (ExchangeComponent *component)
 
 BONOBO_TYPE_FUNC_FULL (ExchangeComponent, GNOME_Evolution_Component, PARENT_TYPE, exchange_component)
 
+void
+exchange_component_set_offline_listener (ExchangeComponent *component, 
+					 ExchangeOfflineListener *listener)
+{
+	ExchangeComponentPrivate *priv;
+
+	g_return_if_fail (EXCHANGE_IS_COMPONENT (component));
+
+	priv = component->priv;
+	priv->offline_listener = listener;
+}
+
 ExchangeComponent *
 exchange_component_new (void)
 {
@@ -397,11 +415,16 @@ exchange_component_get_account_for_uri (ExchangeComponent *component,
 		baccount = acc->data;
 
 		/* Kludge for while we don't support multiple accounts */
-		if (!uri)
-			return baccount->account;
+		if (!exchange_is_offline (priv->offline_listener)) {
+			if (!uri)
+				return baccount->account;
 
-		if (exchange_account_get_folder (baccount->account, uri))
+			if (exchange_account_get_folder (baccount->account, uri))
+				return baccount->account;
+		} else {
+			/* FIXME : Handle multiple accounts */
 			return baccount->account;
+		}
 	}
 	return NULL;
 }
