@@ -150,57 +150,12 @@ setup_addressbook_factory (void)
         return TRUE;
 }
 
-static void
-set_online_status (gboolean is_offline)
-{
-#ifdef OFFLINE_SUPPORT
-        if (is_offline) {
-                e_data_cal_factory_set_backend_mode (cal_factory, CAL_MODE_LOCAL);
-                e_data_book_factory_set_backend_mode (book_factory, CAL_MODE_LOCAL);
-
-        } else {
-                e_data_cal_factory_set_backend_mode (cal_factory, CAL_MODE_REMOTE);
-                e_data_book_factory_set_backend_mode (book_factory, CAL_MODE_REMOTE);
-        }
-#endif
-}
-
-static void
-online_status_changed (GConfClient *client, gint cnxn_id, GConfEntry *entry, gpointer user_data)
-{
-        GConfValue *value;
-        gboolean offline;
-
-        offline = FALSE;
-        value = gconf_entry_get_value (entry);
-        if (value)
-                offline = gconf_value_get_bool (value);
-        set_online_status (offline);
-
-}
-
-static gboolean
-setup_offline_listener ()
-{
-	GConfClient* default_client;
-        GConfValue *value;
-
-        default_client = gconf_client_get_default ();
-        gconf_client_add_dir (default_client, "/apps/evolution/shell", 
-					GCONF_CLIENT_PRELOAD_RECURSIVE,NULL);
-        gconf_client_notify_add (default_client, 
-					"/apps/evolution/shell/start_offline",
-					online_status_changed, NULL, NULL, NULL);
-        value = gconf_client_get (default_client, 
-				"/apps/evolution/shell/start_offline", NULL);
-        set_online_status (gconf_value_get_bool (value));
-        return TRUE;
-}
-
 int
 main (int argc, char **argv)
 {
 	char *path;
+	char *config_directory;
+	ExchangeOfflineListener *offline_listener;
 
 	bindtextdomain (PACKAGE, CONNECTOR_LOCALEDIR);
 	bind_textdomain_codeset (PACKAGE, "UTF-8");
@@ -211,7 +166,10 @@ main (int argc, char **argv)
 			    GNOME_PARAM_HUMAN_READABLE_NAME, _("Ximian Connector for Microsoft Exchange"),
 			    NULL);
 	e_icon_factory_init ();
-	camel_init (NULL, FALSE);
+
+	config_directory = g_build_filename (g_get_home_dir(), ".evolution", NULL);
+	camel_init (config_directory, FALSE);
+	g_free(config_directory);
 
 	path = g_strdup_printf ("/tmp/.exchange-%s", g_get_user_name ());
 	if (mkdir (path, 0700) == -1) {
@@ -242,10 +200,13 @@ main (int argc, char **argv)
         if (!setup_addressbook_factory ())
 		goto failed;
 
-#ifdef OFFLINE_SUPPORT
-	if (!setup_offline_listener ())
+	offline_listener = exchange_offline_listener_new (book_factory, cal_factory);
+	if (!offline_listener) 
 		goto failed;
-#endif
+
+	exchange_component_set_offline_listener (global_exchange_component, 
+						offline_listener);
+	/* FIXME : unref offline_listener here as well ? */
 
 	fprintf (stderr, "Evolution Exchange Storage up and running\n");
 #ifdef E2K_DEBUG
